% simTurn(newAngle, v, dt, TC, init) 
%
% calculate how a simulated forwarder moves for a given steering angle 
% and plots the the vehicle at its new position.
%
% newAngle: The steering angle in radians
% v:  Velocity (m/s)
% dt: delta Time (s)
% TC: 1 if a turning circle will be ploted, else 0
% init: 1 if the initialitation phase, else 0
% record: 1 if a new path should be recorded, else 0
%
function simTurn(newAngle, v, dt, TC, init, record)
global thePath; % defines the path
global vehicle; % Data on the vehicles position...
persistent hdls fLen rLen lf lr wf wr space circ thetar phiF phiR
if(init == 1)    
    lf=5.446; 	% Length of front part of the vehicle
    lr=5.977;	% Length of rear part of the vehicle
    wf=2.995;	% Width of front part of the vehicle
    wr=2.995;	% Width of rear part of the vehicle
    if(record == 1)
        firstSegment = 0;        
    else
        firstSegment = [thePath.x(2) thePath.y(2)] - [thePath.x(1) thePath.y(1)];
        vehicle.x = thePath.x(1); vehicle.y = thePath.y(1); %position of the joint
    end
    space = lf/4; % space between front and rear
    fLen = space/2 + lf/2; %distance between joint and front weel axes
    rLen = space/2 + lr/2; %distance between joint and rear weel axes
    vehicle.length = fLen;
    vehicle.minRadius = calcRadius(deg2rad(20), vehicle.length); % minimum turning radius ( steering angle) 
    vehicle.sa = newAngle/2; 
    r(1) = calcRadius(vehicle.sa, fLen); % front turning radius
    r(2) = calcRadius(vehicle.sa, rLen); % rear turning radius
    o(1) = 2*pi*r(1); %circumference
    o(2) = 2*pi*r(2); %circumference
    deltaAngle(1) = 2*pi*(v*dt/o(1)); %how far it will move each timestep
    deltaAngle(2) = 2*pi*(v*dt/o(2)); %how far it will move each timestep
    
    % set the tractor on track from the beginning
    if(record == 0)
        [vehicle.or, r(1)] = cart2pol(firstSegment(1),firstSegment(2)); %orientation
    end
    thetar = vehicle.or;
    phiF = -vehicle.sa + vehicle.or;
    phiR =  vehicle.sa + thetar;
    [circ(1), circ(2)] = pol2cart(vehicle.or+pi/2, r(1)); % centerpoint of the turning circle 
    
    % coordinates for front and rear end
    [xf,yf] = pol2cart(vehicle.or, lf/2+space/2);
    [xr,yr] = pol2cart(vehicle.or, -(lr/2 + space/2));
    front =[vehicle.x, vehicle.y] + [xf,yf];
    rear = [vehicle.x, vehicle.y] + [xr,yr];
    
    hdls.midPoint=rectangle('Position',[[(vehicle.x-space/2) (vehicle.y-space/2)],space,space],...
        'Curvature',[1,1],'EdgeColor','g','FaceColor','g'); % plot the joint 
    set(hdls.midPoint,'erasemode','xor');     
    if(TC == 1)
        hdls.circ1=rectangle('Position',[circ-abs(r(1)),2*abs(r(1)),2*abs(r(1))],...
            'Curvature',[1,1],'EdgeColor','g'); % plot the turning circle
        set(hdls.circ1,'erasemode','xor');  
    end      
    [cornF, hdls.rekt1] = plotta(lf,wf,phiF,front(1),front(2),1,0,'r'); %front
    [cornR, hdls.rekt2] = plotta(lr,wr,phiR,rear(1),rear(2),1,0,'b'); %rear
    vehicle.corners = [cornF; cornR]; %the corners of the vehicle
else %loop
    trn = min(abs(vehicle.sa-newAngle/2), deg2rad(10)*dt); %turn max 10 deg every timestep
    %     DIFF = rad2deg(abs(vehicle.sa-newAngle/2))%SPR
%     NA = rad2deg(newAngle) %SPR
    if(newAngle/2 < vehicle.sa ) %turn right
        vehicle.sa = max(min(vehicle.sa-trn, deg2rad(20)),  deg2rad(-20));
    else
        vehicle.sa = max(min(vehicle.sa+trn, deg2rad(20)),  deg2rad(-20));
   end
%      SA = rad2deg(vehicle.sa) %SPR
%     TRN = rad2deg(trn) %SPR
    r(1) = calcRadius(vehicle.sa, fLen);
    r(2) = calcRadius(vehicle.sa, rLen);
    o(1) = 2*pi*r(1); %circumference
    o(2) = 2*pi*r(2); %circumference
    deltaAngle(1) = 2*pi*(v*dt/o(1)); %how far it will move each timestep
    deltaAngle(2) = 2*pi*(v*dt/o(2)); %how far it will move each timestep
    [circ(1), circ(2)] = pol2cart(vehicle.or+pi/2, r(1)); % centerpoint of the turning circle
    circ = circ + [vehicle.x,vehicle.y];
    phiF = vehicle.sa + vehicle.or;
    phiR = -vehicle.sa + vehicle.or;
    if(vehicle.sa > -1e-3 & vehicle.sa < 1e-3) % straight ahead
        vehicle.x = vehicle.x + cos(vehicle.or)*(v*dt);
        vehicle.y = vehicle.y + sin(vehicle.or)*(v*dt);
    else
        vehicle.or = mod(vehicle.or + deltaAngle(1)+2*pi, 2*pi);
        thetar = thetar + deltaAngle(2);
        vehicle.x = circ(1) - r(1) * cos(vehicle.or+pi/2);
        vehicle.y = circ(2) - r(1) * sin(vehicle.or+pi/2);
        phiF = phiF + deltaAngle(1);
        phiR = phiR + deltaAngle(2);
    end   
    [xf,yf] = pol2cart(vehicle.or + vehicle.sa, lf/2+space/2);
    [xr,yr] = pol2cart(vehicle.or + (pi-vehicle.sa), lr/2+space/2);
    front = [vehicle.x, vehicle.y] + [xf,yf];
    rear  = [vehicle.x, vehicle.y] + [xr,yr];
    
    [cornF, hdls.rekt1] = plotta(lf,wf,phiF,front(1),front(2),0, hdls.rekt1, 'r'); % front
    [cornR, hdls.rekt2] = plotta(lr,wr,phiR,rear(1),rear(2),0, hdls.rekt2, 'b'); %rear
    vehicle.corners = [cornF; cornR]; %the corners of the vehicle
    
    set(hdls.midPoint,'Position',[[(vehicle.x-space/2), (vehicle.y-space/2)],space,space]) % joint
    if(TC == 1)
        set(hdls.circ1,'Position',[circ-abs(r(1)),2*abs(r(1)),2*abs(r(1))]) % plot the turning circle    
    end
    pause(0.01);    
end